/* Copyright (c) 2016 SpaceToad and the BuildCraft team
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/. */
package buildcraft.lib.block;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.Mirror;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.World;

import buildcraft.api.properties.BuildCraftProperties;

import buildcraft.lib.registry.CreativeTabManager;
import buildcraft.lib.registry.TagManager;
import buildcraft.lib.registry.TagManager.EnumTagType;

public class BlockBCBase_Neptune extends Block {
    public static final IProperty<EnumFacing> PROP_FACING = BuildCraftProperties.BLOCK_FACING;
    public static final IProperty<EnumFacing> BLOCK_FACING_6 = BuildCraftProperties.BLOCK_FACING_6;

    /** The tag used to identify this in the {@link TagManager}. Note that this may be empty if this block doesn't use
     * the tag system. */
    public final String id;

    /** @param id The ID that will be looked up in the {@link TagManager} when registering blocks. Pass null or the
     *            empty string to bypass the {@link TagManager} entirely. */
    public BlockBCBase_Neptune(Material material, String id) {
        super(material);
        if (id == null) {
            id = "";
        }
        this.id = id;

        // Sensible default block properties
        func_149711_c(5.0F);
        func_149752_b(10.0F);
        func_149672_a(SoundType.field_185852_e);

        if (!id.isEmpty()) {
            // Init names from the tag manager
            func_149663_c(TagManager.getTag(id, EnumTagType.UNLOCALIZED_NAME));
            setRegistryName(TagManager.getTag(id, EnumTagType.REGISTRY_NAME));
            func_149647_a(CreativeTabManager.getTab(TagManager.getTag(id, EnumTagType.CREATIVE_TAB)));
        }

        if (this instanceof IBlockWithFacing) {
            IProperty<EnumFacing> facingProp = ((IBlockWithFacing) this).getFacingProperty();
            func_180632_j(func_176223_P().func_177226_a(facingProp, EnumFacing.NORTH));
        }
    }

    // IBlockState

    protected void addProperties(List<IProperty<?>> properties) {
        if (this instanceof IBlockWithFacing) {
            properties.add(((IBlockWithFacing) this).getFacingProperty());
        }
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        List<IProperty<?>> properties = new ArrayList<>();
        addProperties(properties);
        return new BlockStateContainer(this, properties.toArray(new IProperty<?>[0]));
    }

    @Override
    public int func_176201_c(IBlockState state) {
        int meta = 0;
        if (this instanceof IBlockWithFacing) {
            if (((IBlockWithFacing) this).canFaceVertically()) {
                meta |= state.func_177229_b(((IBlockWithFacing) this).getFacingProperty()).func_176745_a();
            } else {
                meta |= state.func_177229_b(((IBlockWithFacing) this).getFacingProperty()).func_176736_b();
            }
        }
        return meta;
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        IBlockState state = func_176223_P();
        if (this instanceof IBlockWithFacing) {
            IBlockWithFacing b = (IBlockWithFacing) this;
            IProperty<EnumFacing> prop = b.getFacingProperty();
            if (b.canFaceVertically()) {
                state = state.func_177226_a(prop, EnumFacing.func_82600_a(meta & 7));
            } else {
                state = state.func_177226_a(prop, EnumFacing.func_176731_b(meta & 3));
            }
        }
        return state;
    }

    @Override
    public IBlockState func_185499_a(IBlockState state, Rotation rot) {
        if (this instanceof IBlockWithFacing) {
            IProperty<EnumFacing> prop = ((IBlockWithFacing) this).getFacingProperty();
            EnumFacing facing = state.func_177229_b(prop);
            state = state.func_177226_a(prop, rot.func_185831_a(facing));
        }
        return state;
    }

    @Override
    public IBlockState func_185471_a(IBlockState state, Mirror mirror) {
        if (this instanceof IBlockWithFacing) {
            IProperty<EnumFacing> prop = ((IBlockWithFacing) this).getFacingProperty();
            EnumFacing facing = state.func_177229_b(prop);
            state = state.func_177226_a(prop, mirror.func_185803_b(facing));
        }
        return state;
    }

    // Others

    @Override
    public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing facing, float hitX, float hitY,
        float hitZ, int meta, EntityLivingBase placer, EnumHand hand) {
        IBlockState state = super.getStateForPlacement(world, pos, facing, hitX, hitY, hitZ, meta, placer, hand);
        if (this instanceof IBlockWithFacing) {
            EnumFacing orientation = placer.func_174811_aO();
            IBlockWithFacing b = (IBlockWithFacing) this;
            if (b.canFaceVertically()) {
                if (MathHelper.func_76135_e((float) placer.field_70165_t - pos.func_177958_n()) < 2.0F
                    && MathHelper.func_76135_e((float) placer.field_70161_v - pos.func_177952_p()) < 2.0F) {
                    double y = placer.field_70163_u + placer.func_70047_e();

                    if (y - pos.func_177956_o() > 2.0D) {
                        orientation = EnumFacing.DOWN;
                    }

                    if (pos.func_177956_o() - y > 0.0D) {
                        orientation = EnumFacing.UP;
                    }
                }
            }
            state = state.func_177226_a(b.getFacingProperty(), orientation.func_176734_d());
        }
        return state;
    }

    @Override
    public boolean rotateBlock(World world, BlockPos pos, EnumFacing axis) {
        if (this instanceof IBlockWithFacing) {
            if (!((IBlockWithFacing) this).canBeRotated(world, pos, world.func_180495_p(pos))) {
                return false;
            }
        }
        return super.rotateBlock(world, pos, axis);
    }

    public static boolean isExceptBlockForAttachWithPiston(Block attachBlock) {
        return Block.func_193382_c(attachBlock);
    }
}
