/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.client.guide.parts;

import java.io.IOException;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.texture.PngSizeInfo;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.resources.IResource;
import net.minecraft.util.ResourceLocation;

import buildcraft.api.core.BCLog;
import buildcraft.api.core.render.ISprite;

import buildcraft.lib.client.guide.GuiGuide;
import buildcraft.lib.client.sprite.SpriteAtlas;
import buildcraft.lib.client.sprite.SpriteRaw;

public class GuideImageFactory implements GuidePartFactory {
    private final ISprite sprite;
    private final int srcWidth, srcHeight;
    private final int width, height;

    public GuideImageFactory(String location) {
        this(location, -1, -1);
    }

    public GuideImageFactory(String location, int width, int height) {
        TextureMap textureMap = Minecraft.func_71410_x().func_147117_R();
        TextureAtlasSprite stitched = textureMap.func_110572_b(location);
        if (stitched != textureMap.func_174944_f()) {
            this.sprite = new SpriteAtlas(stitched);
            this.srcWidth = stitched.func_94211_a();
            this.srcHeight = stitched.func_94216_b();
        } else {
            ISprite s;
            int sw, sh;
            ResourceLocation resLoc = new ResourceLocation(location);
            try (IResource resource = Minecraft.func_71410_x().func_110442_L().func_110536_a(resLoc)) {
                PngSizeInfo size = PngSizeInfo.func_188532_a(resource);
                s = new SpriteRaw(resLoc, 0, 0, 1, 1);
                sw = size.field_188533_a;
                sh = size.field_188534_b;
            } catch (IOException io) {
                BCLog.logger.warn("[lib.guide.loader.image] Couldn't load image '" + resLoc + "' because " + io.getMessage());
                stitched = textureMap.func_174944_f();
                s = new SpriteAtlas(stitched);
                sw = stitched.func_94211_a();
                sh = stitched.func_94216_b();
            }
            this.sprite = s;
            this.srcWidth = sw;
            this.srcHeight = sh;
        }
        this.width = width <= 0 ? srcWidth : width;
        this.height = height <= 0 ? srcHeight : height;
    }

    @Override
    public GuideImage createNew(GuiGuide gui) {
        return new GuideImage(gui, sprite, srcWidth, srcHeight, width, height);
    }
}
